"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const child_process_1 = require("child_process");
const path_1 = require("path");
const semver = require("semver");
const ts = require("typescript");
const fs_1 = require("fs");
const promisify = require("util.promisify");
const createRequire = require('create-require');
const url_1 = require("url");
const execP = promisify(child_process_1.exec);
const TEST_DIR = (0, path_1.join)(__dirname, '../tests');
const PROJECT = (0, path_1.join)(TEST_DIR, 'tsconfig.json');
const BIN_PATH = (0, path_1.join)(TEST_DIR, 'node_modules/.bin/ts-node');
const BIN_SCRIPT_PATH = (0, path_1.join)(TEST_DIR, 'node_modules/.bin/ts-node-script');
const SOURCE_MAP_REGEXP = /\/\/# sourceMappingURL=data:application\/json;charset=utf\-8;base64,[\w\+]+=*$/;
// `createRequire` does not exist on older node versions
const testsDirRequire = createRequire((0, path_1.join)(TEST_DIR, 'index.js')); // tslint:disable-line
// Set after ts-node is installed locally
let { register, create, VERSION, createRepl } = {};
// Pack and install ts-node locally, necessary to test package "exports"
/*
before(async function () {
  this.timeout(5 * 60e3)
  rimrafSync(join(TEST_DIR, 'node_modules'))
  await execP(`npm install`, { cwd: TEST_DIR })
  const packageLockPath = join(TEST_DIR, 'package-lock.json')
  existsSync(packageLockPath) && unlinkSync(packageLockPath)
  ;({ register, create, VERSION, createRepl } = testsDirRequire('ts-node'))
})
*/
describe('ts-node', function () {
    const cmd = `"${BIN_PATH}" --project "${PROJECT}"`;
    const cmdNoProject = `"${BIN_PATH}"`;
    this.timeout(10000);
    it('should export all CJS entrypoints', function () {
        // Ensure our package.json "exports" declaration allows `require()`ing all our entrypoints
        // https://github.com/TypeStrong/ts-node/pull/1026
        testsDirRequire.resolve('ts-node');
        // only reliably way to ask node for the root path of a dependency is Path.resolve(require.resolve('ts-node/package'), '..')
        testsDirRequire.resolve('ts-node/package');
        testsDirRequire.resolve('ts-node/package.json');
        // All bin entrypoints for people who need to augment our CLI: `node -r otherstuff ./node_modules/ts-node/dist/bin`
        testsDirRequire.resolve('ts-node/dist/bin');
        testsDirRequire.resolve('ts-node/dist/bin.js');
        testsDirRequire.resolve('ts-node/dist/bin-transpile');
        testsDirRequire.resolve('ts-node/dist/bin-transpile.js');
        testsDirRequire.resolve('ts-node/dist/bin-script');
        testsDirRequire.resolve('ts-node/dist/bin-script.js');
        // Must be `require()`able obviously
        testsDirRequire.resolve('ts-node/register');
        testsDirRequire.resolve('ts-node/register/files');
        testsDirRequire.resolve('ts-node/register/transpile-only');
        testsDirRequire.resolve('ts-node/register/type-check');
        // `node --loader ts-node/esm`
        testsDirRequire.resolve('ts-node/esm');
        testsDirRequire.resolve('ts-node/esm.mjs');
        testsDirRequire.resolve('ts-node/esm/transpile-only');
        testsDirRequire.resolve('ts-node/esm/transpile-only.mjs');
    });
    describe('cli', function () {
        this.slow(1000);
        it('should execute cli', function (done) {
            (0, child_process_1.exec)(`${cmd} tests/hello-world`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('Hello, world!\n');
                return done();
            });
        });
        it('shows usage via --help', function (done) {
            (0, child_process_1.exec)(`${cmdNoProject} --help`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.match(/Usage: ts-node /);
                return done();
            });
        });
        it('shows version via -v', function (done) {
            (0, child_process_1.exec)(`${cmdNoProject} -v`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout.trim()).to.equal('v' + testsDirRequire('ts-node/package').version);
                return done();
            });
        });
        /*
        it('shows version of compiler via -vv', function (done) {
          exec(`${cmdNoProject} -vv`, function (err, stdout) {
            expect(err).to.equal(null)
            expect(stdout.trim()).to.equal(
              `ts-node v${ testsDirRequire('ts-node/package').version }\n` +
              `node ${ process.version }\n` +
              `compiler v${ testsDirRequire('typescript/package').version }`
            )
            return done()
          })
        })
        */
        it('should register via cli', function (done) {
            (0, child_process_1.exec)(`node -r ts-node/register hello-world.ts`, {
                cwd: TEST_DIR
            }, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('Hello, world!\n');
                return done();
            });
        });
        it('should execute cli with absolute path', function (done) {
            (0, child_process_1.exec)(`${cmd} "${(0, path_1.join)(TEST_DIR, 'hello-world')}"`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('Hello, world!\n');
                return done();
            });
        });
        it('should print scripts', function (done) {
            (0, child_process_1.exec)(`${cmd} -pe "import { example } from './tests/complex/index';example()"`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('example\n');
                return done();
            });
        });
        it('should provide registered information globally', function (done) {
            (0, child_process_1.exec)(`${cmd} tests/env`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('object\n');
                return done();
            });
        });
        it('should provide registered information on register', function (done) {
            (0, child_process_1.exec)(`node -r ts-node/register env.ts`, {
                cwd: TEST_DIR
            }, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('object\n');
                return done();
            });
        });
        if (semver.gte(ts.version, '1.8.0')) {
            it('should allow js', function (done) {
                (0, child_process_1.exec)([
                    cmd,
                    '-O "{\\\"allowJs\\\":true}"',
                    '-pe "import { main } from \'./tests/allow-js/run\';main()"'
                ].join(' '), function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    (0, chai_1.expect)(stdout).to.equal('hello world\n');
                    return done();
                });
            });
            it('should include jsx when `allow-js` true', function (done) {
                (0, child_process_1.exec)([
                    cmd,
                    '-O "{\\\"allowJs\\\":true}"',
                    '-pe "import { Foo2 } from \'./tests/allow-js/with-jsx\'; Foo2.sayHi()"'
                ].join(' '), function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    (0, chai_1.expect)(stdout).to.equal('hello world\n');
                    return done();
                });
            });
        }
        it('should eval code', function (done) {
            (0, child_process_1.exec)(`${cmd} -e "import * as m from './tests/module';console.log(m.example('test'))"`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('TEST\n');
                return done();
            });
        });
        it('should import empty files', function (done) {
            (0, child_process_1.exec)(`${cmd} -e "import './tests/empty'"`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('');
                return done();
            });
        });
        it('should throw errors', function (done) {
            (0, child_process_1.exec)(`${cmd} -e "import * as m from './tests/module';console.log(m.example(123))"`, function (err) {
                if (err === null) {
                    return done('Command was expected to fail, but it succeeded.');
                }
                (0, chai_1.expect)(err.message).to.match(new RegExp('TS2345: Argument of type \'(?:number|123)\' ' +
                    'is not assignable to parameter of type \'string\'\\.'));
                return done();
            });
        });
        it('should be able to ignore diagnostic', function (done) {
            (0, child_process_1.exec)(`${cmd} --ignore-diagnostics 2345 -e "import * as m from './tests/module';console.log(m.example(123))"`, function (err) {
                if (err === null) {
                    return done('Command was expected to fail, but it succeeded.');
                }
                (0, chai_1.expect)(err.message).to.match(/TypeError: (?:(?:undefined|foo\.toUpperCase) is not a function|.*has no method \'toUpperCase\')/);
                return done();
            });
        });
        it('should work with source maps', function (done) {
            (0, child_process_1.exec)(`${cmd} tests/throw`, function (err) {
                if (err === null) {
                    return done('Command was expected to fail, but it succeeded.');
                }
                (0, chai_1.expect)(err.message).to.contain([
                    `${(0, path_1.join)(__dirname, '../tests/throw.ts')}:100`,
                    '  bar () { throw new Error(\'this is a demo\') }',
                    '                 ^',
                    'Error: this is a demo'
                ].join('\n'));
                return done();
            });
        });
        it('eval should work with source maps', function (done) {
            (0, child_process_1.exec)(`${cmd} -pe "import './tests/throw'"`, function (err) {
                if (err === null) {
                    return done('Command was expected to fail, but it succeeded.');
                }
                (0, chai_1.expect)(err.message).to.contain([
                    `${(0, path_1.join)(__dirname, '../tests/throw.ts')}:100`,
                    '  bar () { throw new Error(\'this is a demo\') }',
                    '                 ^'
                ].join('\n'));
                return done();
            });
        });
        it('should support transpile only mode', function (done) {
            (0, child_process_1.exec)(`${cmd} --transpile-only -pe "x"`, function (err) {
                if (err === null) {
                    return done('Command was expected to fail, but it succeeded.');
                }
                (0, chai_1.expect)(err.message).to.contain('ReferenceError: x is not defined');
                return done();
            });
        });
        it('should throw error even in transpileOnly mode', function (done) {
            (0, child_process_1.exec)(`${cmd} --transpile-only -pe "console."`, function (err) {
                if (err === null) {
                    return done('Command was expected to fail, but it succeeded.');
                }
                (0, chai_1.expect)(err.message).to.contain('error TS1003: Identifier expected');
                return done();
            });
        });
        it('should pipe into `ts-node` and evaluate', function (done) {
            const cp = (0, child_process_1.exec)(cmd, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('hello\n');
                return done();
            });
            cp.stdin.end("console.log('hello')");
        });
        it('should pipe into `ts-node`', function (done) {
            const cp = (0, child_process_1.exec)(`${cmd} -p`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('true\n');
                return done();
            });
            cp.stdin.end('true');
        });
        it('should pipe into an eval script', function (done) {
            const cp = (0, child_process_1.exec)(`${cmd} --transpile-only -pe "process.stdin.isTTY"`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('undefined\n');
                return done();
            });
            cp.stdin.end('true');
        });
        it('should run REPL when --interactive passed and stdin is not a TTY', function (done) {
            const cp = (0, child_process_1.exec)(`${cmd} --interactive`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('> 123\n' +
                    'undefined\n' +
                    '> ');
                return done();
            });
            cp.stdin.end('console.log("123")\n');
        });
        it('REPL has command to get type information', function (done) {
            const cp = (0, child_process_1.exec)(`${cmd} --interactive`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('> undefined\n' +
                    '> undefined\n' +
                    '> const a: 123\n' +
                    '> ');
                return done();
            });
            cp.stdin.end('\nconst a = 123\n.type a');
        });
        it('should support require flags', function (done) {
            (0, child_process_1.exec)(`${cmd} -r ./tests/hello-world -pe "console.log('success')"`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('Hello, world!\nsuccess\nundefined\n');
                return done();
            });
        });
        it('should support require from node modules', function (done) {
            (0, child_process_1.exec)(`${cmd} -r typescript -e "console.log('success')"`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('success\n');
                return done();
            });
        });
        it('should use source maps with react tsx', function (done) {
            (0, child_process_1.exec)(`${cmd} tests/throw-react-tsx.tsx`, function (err, stdout) {
                (0, chai_1.expect)(err).not.to.equal(null);
                (0, chai_1.expect)(err.message).to.contain([
                    `${(0, path_1.join)(__dirname, '../tests/throw-react-tsx.tsx')}:100`,
                    '  bar () { throw new Error(\'this is a demo\') }',
                    '                 ^',
                    'Error: this is a demo'
                ].join('\n'));
                return done();
            });
        });
        it('should allow custom typings', function (done) {
            (0, child_process_1.exec)(`${cmd} tests/custom-types`, function (err, stdout) {
                (0, chai_1.expect)(err).to.match(/Error: Cannot find module 'does-not-exist'/);
                return done();
            });
        });
        it('should preserve `ts-node` context with child process', function (done) {
            (0, child_process_1.exec)(`${cmd} tests/child-process`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('Hello, world!\n');
                return done();
            });
        });
        it('should import js before ts by default', function (done) {
            (0, child_process_1.exec)(`${cmd} tests/import-order/compiled`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('Hello, JavaScript!\n');
                return done();
            });
        });
        it('should import ts before js when --prefer-ts-exts flag is present', function (done) {
            (0, child_process_1.exec)(`${cmd} --prefer-ts-exts tests/import-order/compiled`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('Hello, TypeScript!\n');
                return done();
            });
        });
        it('should import ts before js when TS_NODE_PREFER_TS_EXTS env is present', function (done) {
            (0, child_process_1.exec)(`${cmd} tests/import-order/compiled`, { env: Object.assign(Object.assign({}, process.env), { TS_NODE_PREFER_TS_EXTS: 'true' }) }, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('Hello, TypeScript!\n');
                return done();
            });
        });
        it('should ignore .d.ts files', function (done) {
            (0, child_process_1.exec)(`${cmd} tests/import-order/importer`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('Hello, World!\n');
                return done();
            });
        });
        describe('issue #884', function () {
            it('should compile', function (done) {
                // TODO disabled because it consistently fails on Windows on TS 2.7
                if (process.platform === 'win32' && semver.satisfies(ts.version, '2.7')) {
                    this.skip();
                }
                else {
                    (0, child_process_1.exec)(`"${BIN_PATH}" --project tests/issue-884/tsconfig.json tests/issue-884`, function (err, stdout) {
                        (0, chai_1.expect)(err).to.equal(null);
                        (0, chai_1.expect)(stdout).to.equal('');
                        return done();
                    });
                }
            });
        });
        describe('issue #986', function () {
            it('should not compile', function (done) {
                (0, child_process_1.exec)(`"${BIN_PATH}" --project tests/issue-986/tsconfig.json tests/issue-986`, function (err, stdout, stderr) {
                    (0, chai_1.expect)(err).not.to.equal(null);
                    (0, chai_1.expect)(stderr).to.contain('Cannot find name \'TEST\''); // TypeScript error.
                    (0, chai_1.expect)(stdout).to.equal('');
                    return done();
                });
            });
            it('should compile with `--files`', function (done) {
                (0, child_process_1.exec)(`"${BIN_PATH}" --files --project tests/issue-986/tsconfig.json tests/issue-986`, function (err, stdout, stderr) {
                    (0, chai_1.expect)(err).not.to.equal(null);
                    (0, chai_1.expect)(stderr).to.contain('ReferenceError: TEST is not defined'); // Runtime error.
                    (0, chai_1.expect)(stdout).to.equal('');
                    return done();
                });
            });
        });
        if (semver.gte(ts.version, '2.7.0')) {
            it('should support script mode', function (done) {
                (0, child_process_1.exec)(`${BIN_SCRIPT_PATH} tests/scope/a/log`, function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    (0, chai_1.expect)(stdout).to.equal('.ts\n');
                    return done();
                });
            });
            it('should read tsconfig relative to realpath, not symlink, in scriptMode', function (done) {
                if ((0, fs_1.lstatSync)((0, path_1.join)(TEST_DIR, 'main-realpath/symlink/symlink.tsx')).isSymbolicLink()) {
                    (0, child_process_1.exec)(`${BIN_SCRIPT_PATH} tests/main-realpath/symlink/symlink.tsx`, function (err, stdout) {
                        (0, chai_1.expect)(err).to.equal(null);
                        (0, chai_1.expect)(stdout).to.equal('');
                        return done();
                    });
                }
                else {
                    this.skip();
                }
            });
        }
        describe('should read ts-node options from tsconfig.json', function () {
            const BIN_EXEC = `"${BIN_PATH}" --project tests/tsconfig-options/tsconfig.json`;
            it('should override compiler options from env', function (done) {
                (0, child_process_1.exec)(`${BIN_EXEC} tests/tsconfig-options/log-options1.js`, {
                    env: Object.assign(Object.assign({}, process.env), { TS_NODE_COMPILER_OPTIONS: '{"typeRoots": ["env-typeroots"]}' })
                }, function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    const { config } = JSON.parse(stdout);
                    (0, chai_1.expect)(config.options.typeRoots).to.deep.equal([(0, path_1.join)(__dirname, '../tests/tsconfig-options/env-typeroots').replace(/\\/g, '/')]);
                    return done();
                });
            });
            it('should use options from `tsconfig.json`', function (done) {
                (0, child_process_1.exec)(`${BIN_EXEC} tests/tsconfig-options/log-options1.js`, function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    const { options, config } = JSON.parse(stdout);
                    (0, chai_1.expect)(config.options.typeRoots).to.deep.equal([(0, path_1.join)(__dirname, '../tests/tsconfig-options/tsconfig-typeroots').replace(/\\/g, '/')]);
                    (0, chai_1.expect)(config.options.types).to.deep.equal(['tsconfig-tsnode-types']);
                    (0, chai_1.expect)(options.pretty).to.equal(undefined);
                    (0, chai_1.expect)(options.skipIgnore).to.equal(false);
                    (0, chai_1.expect)(options.transpileOnly).to.equal(true);
                    (0, chai_1.expect)(options.require).to.deep.equal([(0, path_1.join)(__dirname, '../tests/tsconfig-options/required1.js')]);
                    return done();
                });
            });
            it('should have flags override / merge with `tsconfig.json`', function (done) {
                (0, child_process_1.exec)(`${BIN_EXEC} --skip-ignore --compiler-options "{\\"types\\":[\\"flags-types\\"]}" --require ./tests/tsconfig-options/required2.js tests/tsconfig-options/log-options2.js`, function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    const { options, config } = JSON.parse(stdout);
                    (0, chai_1.expect)(config.options.typeRoots).to.deep.equal([(0, path_1.join)(__dirname, '../tests/tsconfig-options/tsconfig-typeroots').replace(/\\/g, '/')]);
                    (0, chai_1.expect)(config.options.types).to.deep.equal(['flags-types']);
                    (0, chai_1.expect)(options.pretty).to.equal(undefined);
                    (0, chai_1.expect)(options.skipIgnore).to.equal(true);
                    (0, chai_1.expect)(options.transpileOnly).to.equal(true);
                    (0, chai_1.expect)(options.require).to.deep.equal([
                        (0, path_1.join)(__dirname, '../tests/tsconfig-options/required1.js'),
                        './tests/tsconfig-options/required2.js'
                    ]);
                    return done();
                });
            });
            it('should have `tsconfig.json` override environment', function (done) {
                (0, child_process_1.exec)(`${BIN_EXEC} tests/tsconfig-options/log-options1.js`, {
                    env: Object.assign(Object.assign({}, process.env), { TS_NODE_PRETTY: 'true', TS_NODE_SKIP_IGNORE: 'true' })
                }, function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    const { options, config } = JSON.parse(stdout);
                    (0, chai_1.expect)(config.options.typeRoots).to.deep.equal([(0, path_1.join)(__dirname, '../tests/tsconfig-options/tsconfig-typeroots').replace(/\\/g, '/')]);
                    (0, chai_1.expect)(config.options.types).to.deep.equal(['tsconfig-tsnode-types']);
                    (0, chai_1.expect)(options.pretty).to.equal(true);
                    (0, chai_1.expect)(options.skipIgnore).to.equal(false);
                    (0, chai_1.expect)(options.transpileOnly).to.equal(true);
                    (0, chai_1.expect)(options.require).to.deep.equal([(0, path_1.join)(__dirname, '../tests/tsconfig-options/required1.js')]);
                    return done();
                });
            });
        });
        describe('compiler host', function () {
            it('should execute cli', function (done) {
                (0, child_process_1.exec)(`${cmd} --compiler-host tests/hello-world`, function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    (0, chai_1.expect)(stdout).to.equal('Hello, world!\n');
                    return done();
                });
            });
        });
        it('should transpile files inside a node_modules directory when not ignored', function (done) {
            (0, child_process_1.exec)(`${cmdNoProject} --script-mode tests/from-node-modules/from-node-modules`, function (err, stdout, stderr) {
                if (err)
                    return done(`Unexpected error: ${err}\nstdout:\n${stdout}\nstderr:\n${stderr}`);
                (0, chai_1.expect)(JSON.parse(stdout)).to.deep.equal({
                    external: {
                        tsmri: { name: 'typescript-module-required-internally' },
                        jsmri: { name: 'javascript-module-required-internally' },
                        tsmii: { name: 'typescript-module-imported-internally' },
                        jsmii: { name: 'javascript-module-imported-internally' }
                    },
                    tsmie: { name: 'typescript-module-imported-externally' },
                    jsmie: { name: 'javascript-module-imported-externally' },
                    tsmre: { name: 'typescript-module-required-externally' },
                    jsmre: { name: 'javascript-module-required-externally' }
                });
                done();
            });
        });
        describe('should respect maxNodeModulesJsDepth', function () {
            it('for unscoped modules', function (done) {
                (0, child_process_1.exec)(`${cmdNoProject} --script-mode tests/maxnodemodulesjsdepth`, function (err, stdout, stderr) {
                    (0, chai_1.expect)(err).to.not.equal(null);
                    (0, chai_1.expect)(stderr.replace(/\r\n/g, '\n')).to.contain('TSError: ⨯ Unable to compile TypeScript:\n' +
                        "other.ts(4,7): error TS2322: Type 'string' is not assignable to type 'boolean'.\n" +
                        '\n');
                    done();
                });
            });
            it('for @scoped modules', function (done) {
                (0, child_process_1.exec)(`${cmdNoProject} --script-mode tests/maxnodemodulesjsdepth-scoped`, function (err, stdout, stderr) {
                    (0, chai_1.expect)(err).to.not.equal(null);
                    (0, chai_1.expect)(stderr.replace(/\r\n/g, '\n')).to.contain('TSError: ⨯ Unable to compile TypeScript:\n' +
                        "other.ts(7,7): error TS2322: Type 'string' is not assignable to type 'boolean'.\n" +
                        '\n');
                    done();
                });
            });
        });
    });
    describe('esm', () => {
        this.slow(1000);
        const cmd = `node --loader ts-node/esm`;
        if (semver.gte(process.version, '13.0.0')) {
            it('should compile and execute as ESM', (done) => {
                (0, child_process_1.exec)(`${cmd} index.ts`, { cwd: (0, path_1.join)(__dirname, '../tests/esm') }, function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    (0, chai_1.expect)(stdout).to.equal('foo bar baz biff libfoo\n');
                    return done();
                });
            });
            it('should use source maps', function (done) {
                (0, child_process_1.exec)(`${cmd} throw.ts`, { cwd: (0, path_1.join)(__dirname, '../tests/esm') }, function (err, stdout) {
                    (0, chai_1.expect)(err).not.to.equal(null);
                    (0, chai_1.expect)(err.message).to.contain([
                        `${(0, url_1.pathToFileURL)((0, path_1.join)(__dirname, '../tests/esm/throw.ts'))}:100`,
                        '  bar () { throw new Error(\'this is a demo\') }',
                        '                 ^',
                        'Error: this is a demo'
                    ].join('\n'));
                    return done();
                });
            });
            describe('supports experimental-specifier-resolution=node', () => {
                it('via --experimental-specifier-resolution', (done) => {
                    (0, child_process_1.exec)(`${cmd} --experimental-specifier-resolution=node index.ts`, { cwd: (0, path_1.join)(__dirname, '../tests/esm-node-resolver') }, function (err, stdout) {
                        (0, chai_1.expect)(err).to.equal(null);
                        (0, chai_1.expect)(stdout).to.equal('foo bar baz biff libfoo\n');
                        return done();
                    });
                });
                it('via --es-module-specifier-resolution alias', (done) => {
                    (0, child_process_1.exec)(`${cmd} --experimental-modules --es-module-specifier-resolution=node index.ts`, { cwd: (0, path_1.join)(__dirname, '../tests/esm-node-resolver') }, function (err, stdout) {
                        (0, chai_1.expect)(err).to.equal(null);
                        (0, chai_1.expect)(stdout).to.equal('foo bar baz biff libfoo\n');
                        return done();
                    });
                });
                it('via NODE_OPTIONS', (done) => {
                    (0, child_process_1.exec)(`${cmd} index.ts`, {
                        cwd: (0, path_1.join)(__dirname, '../tests/esm-node-resolver'),
                        env: Object.assign(Object.assign({}, process.env), { NODE_OPTIONS: '--experimental-specifier-resolution=node' })
                    }, function (err, stdout) {
                        (0, chai_1.expect)(err).to.equal(null);
                        (0, chai_1.expect)(stdout).to.equal('foo bar baz biff libfoo\n');
                        return done();
                    });
                });
            });
            it('throws ERR_REQUIRE_ESM when attempting to require() an ESM script while ESM loader is enabled', function (done) {
                (0, child_process_1.exec)(`${cmd} ./index.js`, { cwd: (0, path_1.join)(__dirname, '../tests/esm-err-require-esm') }, function (err, stdout, stderr) {
                    (0, chai_1.expect)(err).to.not.equal(null);
                    (0, chai_1.expect)(stderr).to.contain('Error [ERR_REQUIRE_ESM]: Must use import to load ES Module:');
                    return done();
                });
            });
            it('defers to fallback loaders when URL should not be handled by ts-node', function (done) {
                (0, child_process_1.exec)(`${cmd} index.mjs`, {
                    cwd: (0, path_1.join)(__dirname, '../tests/esm-import-http-url')
                }, function (err, stdout, stderr) {
                    (0, chai_1.expect)(err).to.not.equal(null);
                    // expect error from node's default resolver
                    (0, chai_1.expect)(stderr).to.match(/Error \[ERR_UNSUPPORTED_ESM_URL_SCHEME\]:.*(?:\n.*){0,1}\n *at defaultResolve/);
                    return done();
                });
            });
            it('should bypass import cache when changing search params', (done) => {
                (0, child_process_1.exec)(`${cmd} index.ts`, { cwd: (0, path_1.join)(__dirname, '../tests/esm-import-cache') }, function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    (0, chai_1.expect)(stdout).to.equal('log1\nlog2\nlog2\n');
                    return done();
                });
            });
            it('should support transpile only mode via dedicated loader entrypoint', (done) => {
                (0, child_process_1.exec)(`${cmd}/transpile-only index.ts`, { cwd: (0, path_1.join)(__dirname, '../tests/esm-transpile-only') }, function (err, stdout) {
                    (0, chai_1.expect)(err).to.equal(null);
                    (0, chai_1.expect)(stdout).to.equal('');
                    return done();
                });
            });
            it('should throw type errors without transpile-only enabled', (done) => {
                (0, child_process_1.exec)(`${cmd} index.ts`, { cwd: (0, path_1.join)(__dirname, '../tests/esm-transpile-only') }, function (err, stdout) {
                    if (err === null) {
                        return done('Command was expected to fail, but it succeeded.');
                    }
                    (0, chai_1.expect)(err.message).to.contain('Unable to compile TypeScript');
                    (0, chai_1.expect)(err.message).to.match(new RegExp('TS2345: Argument of type \'(?:number|1101)\' is not assignable to parameter of type \'string\'\\.'));
                    (0, chai_1.expect)(err.message).to.match(new RegExp('TS2322: Type \'(?:"hello world"|string)\' is not assignable to type \'number\'\\.'));
                    (0, chai_1.expect)(stdout).to.equal('');
                    return done();
                });
            });
        }
        it('executes ESM as CJS, ignoring package.json "types" field (for backwards compatibility; should be changed in next major release to throw ERR_REQUIRE_ESM)', function (done) {
            (0, child_process_1.exec)(`${BIN_PATH} ./tests/esm-err-require-esm/index.js`, function (err, stdout) {
                (0, chai_1.expect)(err).to.equal(null);
                (0, chai_1.expect)(stdout).to.equal('CommonJS\n');
                return done();
            });
        });
    });
});
//# sourceMappingURL=index.spec.js.map